// voronoi.cpp
// MCS 481 project 3 description version 1.0
// David Dumas
//
// This is a modified version of a CGAL demo:
//   examples/Voronoi_diagram_2/vd_2_point_location.cpp

#include <iostream>

// includes for defining the Voronoi diagram adaptor
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Delaunay_triangulation_2.h>
#include <CGAL/Voronoi_diagram_2.h>
#include <CGAL/Delaunay_triangulation_adaptation_traits_2.h>
#include <CGAL/Delaunay_triangulation_adaptation_policies_2.h>

// typedefs for defining the adaptor
typedef CGAL::Exact_predicates_inexact_constructions_kernel                  K;
typedef CGAL::Delaunay_triangulation_2<K>                                    DT;
typedef CGAL::Delaunay_triangulation_adaptation_traits_2<DT>                 AT;
typedef CGAL::Delaunay_triangulation_caching_degeneracy_removal_policy_2<DT> AP;
typedef CGAL::Voronoi_diagram_2<DT,AT,AP>                                    VD;

typedef AT::Site_2                    Site_2;
typedef VD::Edge_iterator   Edge_iterator;

int main(int argc, char **argv)
{
  bool quiet = false;

  if ((argc > 1) && (!strcmp(argv[1],"-q"))) {
    quiet = true;
  }
  
  // Construct the voronoi diagram, reading sites from cin/stdin

  VD vd;
  Site_2 t;
  while ( std::cin >> t ) { vd.insert(t); }

  if (!quiet) {
    int ns = 0;
    int nr = 0;
    Edge_iterator eit;
    for (eit = vd.edges_begin(); eit != vd.edges_end(); ++eit) {
      if (eit->is_ray())
	nr++;
      else
	ns++;
    }

    std::cout << "Constructed voronoi diagram:" << std::endl;
    std::cout << vd.number_of_faces() << " sites" << std::endl;
    std::cout << vd.number_of_halfedges()/2 << " edges total" << std::endl;
    std::cout << ns << " bounded edges" << std::endl;
    std::cout << nr << " unbounded edges" << std::endl;
    std::cout << vd.number_of_vertices() << " vertices" << std::endl;
  }

  return 0;
}
